package com.perforce.hws.testing;

import org.eclipse.jetty.server.Server;
import org.eclipse.jetty.servlet.FilterHolder;
import org.eclipse.jetty.servlet.ServletContextHandler;
import org.eclipse.jetty.servlet.ServletHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import spark.servlet.SparkFilter;

import javax.servlet.DispatcherType;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.EnumSet;

/**
 * This is a simple Jetty server that mounts HelixWebServices as a servlet under
 * the path /webservices, and another basic HelloWorld connector under the /hello
 * path.
 */
public class ServletContainer {

    private static final Logger LOGGER = LoggerFactory.getLogger(ServletContainer.class);

    private Server server;

    public ServletContainer() {
        server = new Server(9001);
    }

    public static void main(final String[] args) throws Exception {
        try {
            ServletContainer servletContainer = new ServletContainer();
            servletContainer.start();
            servletContainer.join();

        } catch (Exception e) {
            LOGGER.error("Halting due to error", e);
            throw e;
        }
    }

    public void start() {
        try {
            LOGGER.info("initializing servlet container");

            FilterHolder filterHolder = new FilterHolder(new SparkFilter());

            filterHolder.setInitParameter("applicationClass",
                    "com.perforce.hws.server.HelixWebServices");

            ServletContextHandler contextHandler = new ServletContextHandler();
            contextHandler.setContextPath("/");
            contextHandler.addServlet(HelloServlet.class, "/hello");
            contextHandler.addFilter(filterHolder, "/webservices/*",
                    EnumSet.of(DispatcherType.REQUEST));

            server.setHandler(contextHandler);

            server.start();
            server.dumpStdErr();

            LOGGER.info("servlet container ready");
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void stop() {
        try {
            server.stop();
        } catch (Exception e) {
            throw new IllegalStateException(e);
        }
    }

    public void join() {
        try {
            server.join();
        } catch (InterruptedException e) {
            throw new IllegalStateException(e);
        }
    }


    public static class HelloServlet extends HttpServlet {

        @Override
        protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
            resp.addHeader("Content-Type", "text/html");
            resp.setStatus(HttpServletResponse.SC_OK);
            resp.getWriter().println("<h1>Hello!</h1>");
        }
    }
}
